@@ -19,32 +19,23 @@ void
 CodeEditor::initUI(QFont& font)
 {
     setFont(font);
-
-    setTabStopWidth(QFontMetrics(font).boundingRect("  ").width());
+    QFontMetrics metrics(font);
+    setTabStopDistance(metrics.horizontalAdvance(' ') * indentSize);
 
     lineNumbers= new LineNumbers(this);
     debugInfo = new DebugInfo(this);
     editorHeader = new EditorHeader(this);
     debugInfo->hide();
-    connect(this, SIGNAL(blockCountChanged(int)), this, SLOT(setViewportWidth(int)));
-    connect(this, SIGNAL(updateRequest(QRect,int)), this, SLOT(setLineNumbers(QRect,int)));
-    connect(this, SIGNAL(updateRequest(QRect,int)), this, SLOT(setDebugInfoPos(QRect,int)));
-    connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(cursorChange()));
-    connect(document(), SIGNAL(modificationChanged(bool)), this, SLOT(docChanged(bool)));
-    connect(document(), SIGNAL(contentsChanged()), this, SLOT(contentsChanged()));
+    connect(this, &QPlainTextEdit::blockCountChanged, this, &CodeEditor::setViewportWidth);
+    connect(this, &QPlainTextEdit::updateRequest, this, &CodeEditor::setLineNumbers);
+    connect(this, &QPlainTextEdit::updateRequest, this, &CodeEditor::setDebugInfoPos);
+    connect(this, &QPlainTextEdit::cursorPositionChanged, this, &CodeEditor::cursorChange);
+    connect(document(), &QTextDocument::modificationChanged, this, &CodeEditor::docChanged);
+    connect(document(), &QTextDocument::contentsChanged, this, &CodeEditor::contentsChanged);
 
     setViewportWidth(0);
     cursorChange();
 
-// Default MiniZinc theme
-//    currentTheme = Theme(ThemeColor(Qt::black, Qt::white), // text
-//                         ThemeColor(Qt::darkGreen, QColor(0xbb86fc)), //keywords
-//                         ThemeColor(Qt::blue, QColor(0x13C4F5)), // function
-//                         ThemeColor(Qt::darkRed, QColor(0xF29F05)), // string
-//                         ThemeColor(Qt::red, QColor(0x52514C)), //comment
-//                         ThemeColor(Qt::white,QColor(0x181820)), //background
-//                         ThemeColor(QColor(0xF0F0F0),QColor(0x181820)) // line highlight
-//                         );
     highlighter = new Highlighter(font,darkMode,document());
     setDarkMode(darkMode);
 
@@ -56,8 +47,10 @@ CodeEditor::initUI(QFont& font)
 }
 
 CodeEditor::CodeEditor(QTextDocument* doc, const QString& path, bool isNewFile, bool large,
-                       QFont& font, bool darkMode0, QTabWidget* t, QWidget *parent) :
-    QPlainTextEdit(parent), loadContentsButton(nullptr), tabs(t), darkMode(darkMode0)
+                       QFont& font, int indentSize0, bool useTabs0, bool darkMode0,
+                       QTabWidget* t, QWidget *parent) :
+    QPlainTextEdit(parent), loadContentsButton(nullptr), tabs(t),
+    indentSize(indentSize0), useTabs(useTabs0), darkMode(darkMode0)
 {
     if (doc) {
         QPlainTextEdit::setDocument(doc);
@@ -73,7 +66,7 @@ CodeEditor::CodeEditor(QTextDocument* doc, const QString& path, bool isNewFile,
     if (large) {
         setReadOnly(true);
         QPushButton* pb = new QPushButton("Big file. Load contents?", this);
-        connect(pb, SIGNAL(clicked()), this, SLOT(loadContents()));
+        connect(pb, &QPushButton::clicked, this, &CodeEditor::loadContents);
         loadContentsButton = pb;
     }
     completer = new QCompleter(this);
@@ -92,13 +85,14 @@ CodeEditor::CodeEditor(QTextDocument* doc, const QString& path, bool isNewFile,
     completer->setWrapAround(false);
     completer->setWidget(this);
     completer->setCompletionMode(QCompleter::PopupCompletion);
-    QObject::connect(completer, SIGNAL(activated(QString)), this, SLOT(insertCompletion(QString)));
+    QObject::connect(completer, QOverload<const QString&>::of(&QCompleter::activated), this, &CodeEditor::insertCompletion);
 
     modificationTimer.setSingleShot(true);
-    QObject::connect(&modificationTimer, SIGNAL(timeout()), this, SLOT(contentsChangedWithTimeout()));
+    QObject::connect(&modificationTimer, &QTimer::timeout, this, &CodeEditor::contentsChangedWithTimeout);
 
-    if (parent) {
-        QObject::connect(this, SIGNAL(escPressed()), parent, SLOT(on_closeFindWidget_clicked()));
+    auto* mw = qobject_cast<MainWindow*>(parent);
+    if (mw != nullptr) {
+        QObject::connect(this, &CodeEditor::escPressed, mw, &MainWindow::closeFindWidget);
     }
 
     setAcceptDrops(false);
@@ -133,11 +127,11 @@ void CodeEditor::setDocument(QTextDocument *document)
         delete highlighter;
         highlighter = nullptr;
     }
-    disconnect(this, SIGNAL(blockCountChanged(int)), this, SLOT(setViewportWidth(int)));
-    disconnect(this, SIGNAL(updateRequest(QRect,int)), this, SLOT(setLineNumbers(QRect,int)));
-    disconnect(this, SIGNAL(updateRequest(QRect,int)), this, SLOT(setDebugInfoPos(QRect,int)));
-    disconnect(this, SIGNAL(cursorPositionChanged()), this, SLOT(cursorChange()));
-    disconnect(this->document(), SIGNAL(modificationChanged(bool)), this, SLOT(docChanged(bool)));
+    disconnect(this, &QPlainTextEdit::blockCountChanged, this, &CodeEditor::setViewportWidth);
+    disconnect(this, &QPlainTextEdit::updateRequest, this, &CodeEditor::setLineNumbers);
+    disconnect(this, &QPlainTextEdit::updateRequest, this, &CodeEditor::setDebugInfoPos);
+    disconnect(this, &QPlainTextEdit::cursorPositionChanged, this, &CodeEditor::cursorChange);
+    disconnect(this->document(), &QTextDocument::modificationChanged, this, &CodeEditor::docChanged);
     QList<QTextEdit::ExtraSelection> noSelections;
     setExtraSelections(noSelections);
     QPlainTextEdit::setDocument(document);
@@ -145,11 +139,11 @@ void CodeEditor::setDocument(QTextDocument *document)
         QFont f= font();
         highlighter = new Highlighter(f,darkMode,document);
     }
-    connect(this, SIGNAL(blockCountChanged(int)), this, SLOT(setViewportWidth(int)));
-    connect(this, SIGNAL(updateRequest(QRect,int)), this, SLOT(setLineNumbers(QRect,int)));
-    connect(this, SIGNAL(updateRequest(QRect,int)), this, SLOT(setDebugInfoPos(QRect,int)));
-    connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(cursorChange()));
-    connect(this->document(), SIGNAL(modificationChanged(bool)), this, SLOT(docChanged(bool)));
+    connect(this, &QPlainTextEdit::blockCountChanged, this, &CodeEditor::setViewportWidth);
+    connect(this, &QPlainTextEdit::updateRequest, this, &CodeEditor::setLineNumbers);
+    connect(this, &QPlainTextEdit::updateRequest, this, &CodeEditor::setDebugInfoPos);
+    connect(this, &QPlainTextEdit::cursorPositionChanged, this, &CodeEditor::cursorChange);
+    connect(this->document(), &QTextDocument::modificationChanged, this, &CodeEditor::docChanged);
 }
 
 void CodeEditor::setDarkMode(bool enable)
@@ -168,10 +162,17 @@ void CodeEditor::setDarkMode(bool enable)
 //    palette.setColor(QPalette::HighlightedText, Themes::currentTheme.textColor.get(darkMode));
 //    this->setPalette(palette);
 
-    setStyleSheet(Themes::currentTheme.styleSheet(darkMode));
+    viewport()->setStyleSheet(Themes::currentTheme.styleSheet(darkMode));
     cursorChange(); // Ensure extra selections are the correct colours
 }
 
+void CodeEditor::setIndentSize(int size)
+{
+    indentSize = size;
+    QFontMetrics metrics(font());
+    setTabStopDistance(metrics.horizontalAdvance(' ') * size);
+}
+
 Highlighter& CodeEditor::getHighlighter() {
     return *highlighter;
 }
@@ -213,19 +214,52 @@ void CodeEditor::keyPressEvent(QKeyEvent *e)
             break;
         }
     }
-    if (e->key() == Qt::Key_Tab) {
+    if (e->key() == Qt::Key_Backtab) {
         e->accept();
-        QTextCursor cursor(textCursor());
-        cursor.insertText("  ");
+        shiftLeft();
+        ensureCursorVisible();
+    } else if (e->key() == Qt::Key_Tab) {
+        e->accept();
+        auto cursor = textCursor();
+        auto startBlock = document()->findBlock(cursor.selectionStart());
+        auto endBlock = document()->findBlock(cursor.selectionEnd());
+        auto partialLineSelection = startBlock.position() != cursor.selectionStart()
+                || endBlock.position() + endBlock.length() - 1 != cursor.selectionEnd();
+        if (!cursor.hasSelection() || (startBlock == endBlock && partialLineSelection)) {
+            if (useTabs) {
+                cursor.insertText("\t");
+            } else {
+                auto posInLine = cursor.selectionStart() - startBlock.position();
+                auto distanceFromTabStop = 0;
+                auto line = startBlock.text();
+                for (int i = 0; i < posInLine; i++) {
+                    if (line[i] == '\t') {
+                        distanceFromTabStop = 0;
+                    } else if (line[i].isSpace()) {
+                        distanceFromTabStop++;
+                    } else {
+                        break;
+                    }
+                }
+                auto toAdd = distanceFromTabStop % indentSize;
+                if (toAdd == 0) {
+                    toAdd = indentSize;
+                }
+                cursor.insertText(QString(" ").repeated(toAdd));
+            }
+        } else {
+            shiftRight();
+        }
         ensureCursorVisible();
     } else if (e->key() == Qt::Key_Return) {
         e->accept();
         QTextCursor cursor(textCursor());
         QString curLine = cursor.block().text();
-        QRegExp leadingWhitespace("^(\\s*)");
+        QRegularExpression leadingWhitespace("^(\\s*)");
         cursor.insertText("\n");
-        if (leadingWhitespace.indexIn(curLine) != -1) {
-            cursor.insertText(leadingWhitespace.cap(1));
+        auto leadingWhitespace_match = leadingWhitespace.match(curLine);
+        if (leadingWhitespace_match.hasMatch()) {
+            cursor.insertText(leadingWhitespace_match.captured(1));
         }
         ensureCursorVisible();
     } else if (e->key() == Qt::Key_Escape) {
@@ -378,8 +412,10 @@ void CodeEditor::cursorChange()
         extraSelections.append(highlightLineSelection);
     }
 
+    auto ec = Themes::currentTheme.errorColor.get(darkMode);
+    auto wc = Themes::currentTheme.warningColor.get(darkMode);
     foreach (QTextEdit::ExtraSelection sel, allExtraSels) {
-        if (sel.format.underlineColor()==Qt::red) {
+        if (sel.format.underlineColor() == ec || sel.format.underlineColor() == wc) {
             extraSelections.append(sel);
         }
     }
@@ -531,6 +567,8 @@ void CodeEditor::paintLineNumbers(QPaintEvent *event)
 
             if (errorLines.contains(blockNumber)) {
                 painter.setPen(Themes::currentTheme.errorColor.get(darkMode));
+            } else if (warningLines.contains(blockNumber)) {
+                painter.setPen(Themes::currentTheme.warningColor.get(darkMode));
             } else if (blockNumber == curLine) {
                 painter.setPen(Themes::currentTheme.foregroundActiveColor.get(darkMode));
             } else {
@@ -652,6 +690,7 @@ void CodeEditor::setEditorFont(QFont& font)
     setFont(font);
     document()->setDefaultFont(font);
     highlighter->setEditorFont(font);
+    setIndentSize(indentSize);
     highlighter->rehighlight();
 }
 
@@ -672,7 +711,6 @@ bool CodeEditor::eventFilter(QObject *, QEvent *ev)
         evPos = QPoint(evPos.x()-lineNumbersWidth(),evPos.y());
         if (evPos.x() >= 0) {
             QTextCursor cursor = cursorForPosition(evPos);
-            cursor.movePosition(QTextCursor::PreviousCharacter);
             bool foundError = false;
             foreach (CodeEditorError cee, errors) {
                 if (cursor.position() >= cee.startPos && cursor.position() <= cee.endPos) {
@@ -716,29 +754,33 @@ void CodeEditor::cut()
 
 void CodeEditor::checkFile(const QVector<MiniZincError>& mznErrors)
 {
+    auto errorColor = Themes::currentTheme.errorColor.get(darkMode);
+    auto warningColor = Themes::currentTheme.warningColor.get(darkMode);
+
     QList<QTextEdit::ExtraSelection> allExtraSels = extraSelections();
 
     QList<QTextEdit::ExtraSelection> extraSelections;
     foreach (QTextEdit::ExtraSelection sel, allExtraSels) {
-        if (sel.format.underlineColor()!=Qt::red) {
+        if (sel.format.underlineColor() != errorColor && sel.format.underlineColor() != warningColor) {
             extraSelections.append(sel);
         }
     }
 
     errors.clear();
     errorLines.clear();
-    for (unsigned int i=0; i<mznErrors.size(); i++) {
+    warningLines.clear();
+    for (auto& it : mznErrors) {
         QTextEdit::ExtraSelection sel;
         QTextCharFormat format = sel.format;
         format.setUnderlineStyle(QTextCharFormat::WaveUnderline);
-        format.setUnderlineColor(Themes::currentTheme.errorColor.get(darkMode));
-        QTextBlock block = document()->findBlockByNumber(mznErrors[i].first_line-1);
-        QTextBlock endblock = document()->findBlockByNumber(mznErrors[i].last_line-1);
+        format.setUnderlineColor(it.isWarning ? warningColor : errorColor);
+        QTextBlock block = document()->findBlockByNumber(it.first_line-1);
+        QTextBlock endblock = document()->findBlockByNumber(it.last_line-1);
         if (block.isValid() && endblock.isValid()) {
             QTextCursor cursor = textCursor();
             cursor.setPosition(block.position());
-            int firstCol = mznErrors[i].first_col < mznErrors[i].last_col ? (mznErrors[i].first_col-1) : (mznErrors[i].last_col-1);
-            int lastCol = mznErrors[i].first_col < mznErrors[i].last_col ? (mznErrors[i].last_col) : (mznErrors[i].first_col);
+            int firstCol = it.first_col < it.last_col ? (it.first_col-1) : (it.last_col-1);
+            int lastCol = it.first_col < it.last_col ? (it.last_col) : (it.first_col);
             cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::MoveAnchor, firstCol);
             int startPos = cursor.position();
             cursor.setPosition(endblock.position(), QTextCursor::KeepAnchor);
@@ -747,12 +789,67 @@ void CodeEditor::checkFile(const QVector<MiniZincError>& mznErrors)
             sel.cursor = cursor;
             sel.format = format;
             extraSelections.append(sel);
-            CodeEditorError cee(startPos,endPos,mznErrors[i].msg);
+            CodeEditorError cee(startPos, endPos, QString(it.isWarning ? "Warning: %1" : "Error: %1").arg(it.msg));
             errors.append(cee);
-            for (int j=mznErrors[i].first_line; j<=mznErrors[i].last_line; j++) {
-                errorLines.insert(j-1);
+            for (int j=it.first_line; j<=it.last_line; j++) {
+                if (it.isWarning) {
+                    warningLines.insert(j - 1);
+                } else {
+                    errorLines.insert(j - 1);
+                }
             }
         }
     }
     setExtraSelections(extraSelections);
 }
+
+void CodeEditor::shiftLeft()
+{
+    shiftSelected(-1);
+}
+
+void CodeEditor::shiftRight()
+{
+    shiftSelected(1);
+}
+
+void CodeEditor::shiftSelected(int amount)
+{
+    auto origCursor = textCursor();
+    origCursor.setKeepPositionOnInsert(true);
+    QTextCursor cursor = textCursor();
+    QTextBlock block = document()->findBlock(cursor.selectionStart());
+    QTextBlock endblock = document()->findBlock(cursor.selectionEnd());
+    bool atBlockStart = cursor.selectionEnd() == endblock.position();
+    if (block==endblock || !atBlockStart)
+        endblock = endblock.next();
+    cursor.beginEditBlock();
+    do {
+        auto position = block.position();
+        cursor.setPosition(position);
+        auto indentation = 0;
+        for (auto c : block.text()) {
+            if (c == '\t') {
+                indentation = indentSize * ((indentation + indentSize) / indentSize);
+            } else if (c.isSpace()) {
+                indentation++;
+            } else {
+                break;
+            }
+            position++;
+        }
+        cursor.setPosition(position, QTextCursor::KeepAnchor);
+        auto newIndent = std::max(0, indentation + indentSize * amount);
+        if (useTabs) {
+            cursor.insertText(
+                        QString("\t").repeated(newIndent / indentSize) +
+                        QString(" ").repeated(newIndent % indentSize));
+        } else {
+            cursor.insertText(QString(" ").repeated(newIndent));
+        }
+        block = block.next();
+    } while (block.isValid() && block != endblock);
+    cursor.endEditBlock();
+    origCursor.setKeepPositionOnInsert(false);
+    setTextCursor(origCursor);
+}
