@@ -21,9 +21,11 @@
 
 #include "process.h"
 #include "exception.h"
+#include "ideutils.h"
 
 #include <algorithm>
 #include <limits>
+#include <cfloat>
 
 ConfigWindow::ConfigWindow(QWidget *parent) :
     QWidget(parent),
@@ -61,14 +63,8 @@ ConfigWindow::ConfigWindow(QWidget *parent) :
     ui->extraParams_tableWidget->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
     ui->extraParams_tableWidget->setItemDelegateForColumn(3, new ExtraOptionDelegate);
 
-    QList<QWidget*> toBeWatched;
-    for (auto child : findChildren<QWidget*>()) {
-        if (child != ui->config_comboBox) {
-            toBeWatched << child;
-        }
-    }
-    watchChanges(toBeWatched, std::bind(&ConfigWindow::invalidate, this, false));
-    watchChanges(ui->options_groupBox->findChildren<QWidget*>(), std::bind(&ConfigWindow::invalidate, this, true));
+    IDEUtils::watchChildChanges(ui->options_groupBox, this, std::bind(&ConfigWindow::invalidate, this, true));
+    IDEUtils::watchChildChanges(ui->advanced_groupBox, this, std::bind(&ConfigWindow::invalidate, this, false));
 }
 
 ConfigWindow::~ConfigWindow()
@@ -83,20 +79,13 @@ void ConfigWindow::init()
 {
     loadConfigs();
     initialized = true;
-    int i = 0;
-    for (auto config : configs) {
-        if (config->solverDefinition.isDefaultSolver) {
-            setCurrentIndex(i);
-            break;
-        }
-        i++;
-    }
 }
 
 void ConfigWindow::loadConfigs(void)
 {
     try {
         QStringList files;
+        QStringList warnings;
         for (auto config : configs) {
             if (!config->paramFile.isEmpty()) {
                 files.append(config->paramFile);
@@ -104,18 +93,21 @@ void ConfigWindow::loadConfigs(void)
             delete config;
         }
         configs.clear();
-        for (auto& solver : MznDriver::get().solvers()) {
-            if (solver.hasAllRequiredFlags()) {
-                configs.append(new SolverConfiguration(solver, true));
+        for (auto* solver : MznDriver::get().solvers()) {
+            if (solver->hasAllRequiredFlags()) {
+                configs.append(new SolverConfiguration(*solver, true));
             }
         }
         if (!files.empty()) {
             for (auto fileName : files) {
-                configs.append(new (SolverConfiguration) (SolverConfiguration::loadJSON(fileName)));
+                configs.append(new (SolverConfiguration) (SolverConfiguration::loadJSON(fileName, warnings)));
             }
         }
         populateComboBox();
         updateGUI(true);
+        if (!warnings.empty()) {
+            QMessageBox::warning(this, "MiniZinc IDE", warnings.join("\n"), QMessageBox::Ok);
+        }
     } catch (Exception& e) {
         QMessageBox::warning(this, "Parameter configuration error", e.message(), QMessageBox::Ok);
     }
@@ -131,8 +123,9 @@ bool ConfigWindow::addConfig(const QString &fileName)
     }
 
     try {
+        QStringList warnings;
         updateSolverConfig(configs[currentIndex()]);
-        configs.append(new (SolverConfiguration) (SolverConfiguration::loadJSON(fileName)));
+        configs.append(new (SolverConfiguration) (SolverConfiguration::loadJSON(fileName, warnings)));
         for (auto sc : configs) {
             if (!sc->syncedOptionsMatch(*(configs.last()))) {
                 populating = true; // Make sure this doesn't mark the config as modified
@@ -142,7 +135,22 @@ bool ConfigWindow::addConfig(const QString &fileName)
             }
         }
         populateComboBox();
-        setCurrentIndex(configs.length() - 1);
+
+        if (ui->sync_checkBox->isChecked()) {
+            // Ensure that newly loaded config overrides synced options
+            populating = true; // Make sure this doesn't mark the config as modified
+            ui->sync_checkBox->setChecked(false);
+            populating = false;
+            setCurrentIndex(configs.length() - 1);
+            populating = true; // Make sure this doesn't mark the config as modified
+            ui->sync_checkBox->setChecked(true);
+            populating = false;
+        }
+
+        if (!warnings.empty()) {
+            QMessageBox::warning(this, "MiniZinc IDE", warnings.join("\n"), QMessageBox::Ok);
+        }
+
         return true;
     } catch (Exception& e) {
         QMessageBox::warning(this, "Parameter configuration error", e.message(), QMessageBox::Ok);
@@ -188,6 +196,10 @@ int ConfigWindow::findBuiltinConfig(const QString &id, const QString &version)
 
 int ConfigWindow::findConfigFile(const QString &file)
 {
+    if (file.isEmpty()) {
+        // Unsaved configuration
+        return -1;
+    }
     int i = 0;
     for (auto sc : configs) {
         if (sc->paramFile == file) {
@@ -198,6 +210,60 @@ int ConfigWindow::findConfigFile(const QString &file)
     return -1;
 }
 
+void ConfigWindow::stashModifiedConfigs()
+{
+    auto* selected = currentSolverConfig();
+    for (auto* sc : configs) {
+        if (sc->modified) {
+            if (sc == selected) {
+                stashSelectedModifiedConfig = stash.length();
+            }
+            stash.append(StashItem(sc));
+        }
+    }
+    if (selected != nullptr) {
+        if (selected->isBuiltin) {
+            stashSelectedBuiltinSolverId = selected->solverDefinition.id;
+            stashSelectedBuiltinSolverVersion = selected->solverDefinition.version;
+        }
+        stashSelectedParamFile = selected->paramFile;
+    }
+}
+
+void ConfigWindow::unstashModifiedConfigs()
+{
+    lastIndex = -1;
+
+    QList<SolverConfiguration*> merge;
+    for (auto& item : stash) {
+        try {
+            merge.append(item.consume());
+        }
+        catch (...) {
+            // Ignore
+        }
+    }
+    mergeConfigs(merge);
+
+    int i = -1;
+    if (stashSelectedModifiedConfig >= 0 && stashSelectedModifiedConfig < merge.size()) {
+        i = configs.indexOf(merge[stashSelectedModifiedConfig]);
+    } else if (!stashSelectedParamFile.isEmpty()) {
+        i = findConfigFile(stashSelectedParamFile);
+    } else if (!stashSelectedBuiltinSolverId.isEmpty()) {
+        i = findBuiltinConfig(stashSelectedBuiltinSolverId, stashSelectedBuiltinSolverVersion);
+    }
+    if (i != -1) {
+        setCurrentIndex(i);
+    }
+
+    stash.clear();
+    stashSelectedBuiltinSolverId = "";
+    stashSelectedBuiltinSolverVersion = "";
+    stashSelectedParamFile = "";
+    stashSelectedModifiedConfig = -1;
+}
+
 QString ConfigWindow::saveConfig(int index)
 {
     auto sc = configs[index];
@@ -343,9 +409,17 @@ void ConfigWindow::updateGUI(bool overrideSync)
 
     ui->reset_pushButton->setEnabled(sc->isBuiltin);
 
+    ui->intermediateSolutions_checkBox->setEnabled(sc->supports("-a") || sc->supports("-i"));
+
+    ui->numSolutions_checkBox->setEnabled(sc->supports("-a"));
+    ui->numOptimal_checkBox->setEnabled(sc->supports("-a-o"));
+    ui->verboseSolving_checkBox->setEnabled(sc->supports("-v"));
+    ui->solvingStats_checkBox->setEnabled(sc->supports("-s"));
+
     if (overrideSync || !ui->sync_checkBox->isChecked()) {
-        ui->timeLimit_doubleSpinBox->setValue(sc->timeLimit / 1000.0);
         ui->timeLimit_checkBox->setChecked(sc->timeLimit != 0);
+        ui->timeLimit_doubleSpinBox->setValue(sc->timeLimit / 1000.0);
+        ui->timeLimit_doubleSpinBox->setEnabled(ui->timeLimit_checkBox->isChecked());
 
         if (sc->printIntermediate && sc->numSolutions == 1) {
             ui->defaultBehavior_radioButton->setChecked(true);
@@ -354,48 +428,40 @@ void ConfigWindow::updateGUI(bool overrideSync)
         }
 
         ui->intermediateSolutions_checkBox->setChecked(sc->printIntermediate);
-
         ui->numSolutions_spinBox->setValue(sc->numSolutions);
         ui->numSolutions_checkBox->setChecked(sc->numSolutions != 0);
-
         ui->numOptimal_spinBox->setValue(sc->numOptimal);
         ui->numOptimal_checkBox->setChecked(sc->numOptimal != 0);
-
         ui->verboseCompilation_checkBox->setChecked(sc->verboseCompilation);
-        ui->verboseSolving_checkBox->setChecked(sc->verboseSolving);
 
+        if (ui->verboseSolving_checkBox->isEnabled()) {
+            ui->verboseSolving_checkBox->setChecked(sc->verboseSolving);
+        }
         ui->compilationStats_checkBox->setChecked(sc->compilationStats);
-        ui->solvingStats_checkBox->setChecked(sc->solvingStats);
+
+        if (ui->solvingStats_checkBox->isEnabled()) {
+            ui->solvingStats_checkBox->setChecked(sc->solvingStats);
+        }
+
         ui->timingInfo_checkBox->setChecked(sc->outputTiming);
     }
 
-    ui->intermediateSolutions_checkBox->setEnabled(
-                sc->solverDefinition.stdFlags.contains("-a") ||
-                sc->solverDefinition.stdFlags.contains("-i"));
-
-    ui->numSolutions_checkBox->setEnabled(sc->solverDefinition.stdFlags.contains("-a"));
-    ui->numSolutions_spinBox->setEnabled(sc->solverDefinition.stdFlags.contains("-n"));
-
-    ui->numOptimal_checkBox->setEnabled(sc->solverDefinition.stdFlags.contains("-a-o"));
-    ui->numOptimal_spinBox->setEnabled(sc->solverDefinition.stdFlags.contains("-n-o"));
-
-    ui->verboseSolving_checkBox->setEnabled(sc->solverDefinition.stdFlags.contains("-v"));
-
-    ui->solvingStats_checkBox->setEnabled(sc->solverDefinition.stdFlags.contains("-s"));
+    ui->numSolutions_spinBox->setEnabled(ui->numSolutions_checkBox->isEnabled() && ui->numSolutions_checkBox->isChecked());
+    ui->numOptimal_spinBox->setEnabled(ui->numOptimal_checkBox->isEnabled() && ui->numOptimal_checkBox->isChecked());
 
     ui->optimizationLevel_comboBox->setCurrentIndex(sc->optimizationLevel);
     ui->additionalData_plainTextEdit->setPlainText(sc->additionalData.join("\n"));
 
-    ui->numThreads_spinBox->setEnabled(sc->solverDefinition.stdFlags.contains("-p"));
+    ui->numThreads_spinBox->setEnabled(sc->supports("-p"));
     ui->numThreads_spinBox->setValue(sc->numThreads);
 
-    bool enableRandomSeed = sc->solverDefinition.stdFlags.contains("-r");
+    bool enableRandomSeed = sc->supports("-r");
     ui->randomSeed_lineEdit->setText(sc->randomSeed.toString());
     ui->randomSeed_checkBox->setChecked(!sc->randomSeed.isNull());
     ui->randomSeed_checkBox->setEnabled(enableRandomSeed);
     ui->randomSeed_lineEdit->setEnabled(enableRandomSeed && !sc->randomSeed.isNull());
 
-    bool enableFreeSearch = sc->solverDefinition.stdFlags.contains("-f");
+    bool enableFreeSearch = sc->supports("-f");
     ui->freeSearch_checkBox->setEnabled(enableFreeSearch);
     ui->freeSearch_checkBox->setChecked(sc->freeSearch);
 
@@ -467,7 +533,7 @@ void ConfigWindow::updateSolverConfig(SolverConfiguration* sc)
     QString additionalData = ui->additionalData_plainTextEdit->toPlainText();
     sc->additionalData = additionalData.length() > 0 ? additionalData.split("\n") : QStringList();
     sc->numThreads = ui->numThreads_spinBox->value();
-    sc->randomSeed = ui->randomSeed_checkBox->isChecked() ? ui->randomSeed_lineEdit->text() : nullptr;
+    sc->randomSeed = ui->randomSeed_checkBox->isChecked() ? ui->randomSeed_lineEdit->text() : QVariant();
     sc->freeSearch = ui->freeSearch_checkBox->isChecked();
 
     sc->extraOptions.clear();
@@ -608,38 +674,6 @@ void ConfigWindow::addExtraParam(const QString& key, bool backend, const QVarian
     ui->extraParams_tableWidget->setCellWidget(i, 2, typeComboBox);
 }
 
-void ConfigWindow::watchChanges(const QList<QWidget*>& widgets, std::function<void()> action)
-{
-    for (auto widget : widgets) {
-        QCheckBox* checkBox;
-        QLineEdit* lineEdit;
-        QSpinBox* spinBox;
-        QDoubleSpinBox* doubleSpinBox;
-        QComboBox* comboBox;
-        QGroupBox* groupBox;
-        QTableWidget* tableWidget;
-        QPlainTextEdit* plainTextEdit;
-
-        if ((checkBox = qobject_cast<QCheckBox*>(widget))) {
-            connect(checkBox, &QCheckBox::stateChanged, [=] (int) { action(); });
-        } else if ((lineEdit = qobject_cast<QLineEdit*>(widget))) {
-            connect(lineEdit, &QLineEdit::textChanged, [=] (const QString&) { action(); });
-        } else if ((spinBox = qobject_cast<QSpinBox*>(widget))) {
-            connect(spinBox, QOverload<int>::of(&QSpinBox::valueChanged), [=] (int) { action(); });
-        } else if ((doubleSpinBox = qobject_cast<QDoubleSpinBox*>(widget))) {
-            connect(doubleSpinBox, QOverload<double>::of(&QDoubleSpinBox::valueChanged), [=] (double) { action(); });
-        } else if ((comboBox = qobject_cast<QComboBox*>(widget))) {
-            connect(comboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), [=] (int) { action(); });
-        } else if ((groupBox = qobject_cast<QGroupBox*>(widget))) {
-            connect(groupBox, &QGroupBox::toggled, [=] (bool) { action(); });
-        } else if ((tableWidget = qobject_cast<QTableWidget*>(widget))) {
-            connect(tableWidget, &QTableWidget::cellChanged, [=] (int, int) { action(); });
-        } else if ((plainTextEdit = qobject_cast<QPlainTextEdit*>(widget))) {
-            connect(plainTextEdit, &QPlainTextEdit::textChanged, [=] () { action(); });
-        }
-    }
-}
-
 void ConfigWindow::invalidate(bool all)
 {
     if (populating) {
@@ -682,7 +716,19 @@ void ConfigWindow::populateComboBox()
     initialized = false;
     ui->config_comboBox->clear();
     ui->config_comboBox->addItems(items);
-    setCurrentIndex(oldIndex);
+    if (oldIndex >= 0 && oldIndex < configs.length()) {
+        setCurrentIndex(oldIndex);
+    } else {
+        // Index no longer valid, just go back to default solver
+        int i = 0;
+        for (auto config : configs) {
+            if (config->solverDefinition.isDefaultSolver) {
+                setCurrentIndex(i);
+                break;
+            }
+            i++;
+        }
+    }
     emit itemsChanged(items);
     initialized = oldInitialized;
 }
@@ -759,6 +805,7 @@ QWidget* ExtraOptionDelegate::createEditor(QWidget* parent, const QStyleOptionVi
         case SolverFlag::T_FLOAT_RANGE:
         {
             auto field = new QDoubleSpinBox(parent);
+            field->setDecimals(DBL_MAX_10_EXP + DBL_DIG);
             field->setRange(f.min, f.max);
             return field;
         }
@@ -771,8 +818,8 @@ QWidget* ExtraOptionDelegate::createEditor(QWidget* parent, const QStyleOptionVi
         case SolverFlag::T_SOLVER:
         {
             auto field = new QComboBox(parent);
-            for (auto& solver : MznDriver::get().solvers()) {
-                field->addItem(solver.name);
+            for (auto* solver : MznDriver::get().solvers()) {
+                field->addItem(solver->name);
             }
             return field;
         }
@@ -781,6 +828,11 @@ QWidget* ExtraOptionDelegate::createEditor(QWidget* parent, const QStyleOptionVi
         case SolverFlag::T_STRING:
             return QStyledItemDelegate::createEditor(parent, option, index);
         }
+    } else if (index.data().type() == QVariant::Double) {
+        auto field = new QLineEdit(parent);
+        field->setAlignment(Qt::AlignRight);
+        field->setValidator(new QDoubleValidator());
+        return field;
     } else {
         return QStyledItemDelegate::createEditor(parent, option, index);
     }
@@ -819,6 +871,8 @@ void ExtraOptionDelegate::setEditorData(QWidget* editor, const QModelIndex& inde
             QStyledItemDelegate::setEditorData(editor, index);
             break;
         }
+    } else if (index.data().type() == QVariant::Double) {
+        static_cast<QLineEdit*>(editor)->setText(QString::number(index.data().toDouble()));
     } else {
         QStyledItemDelegate::setEditorData(editor, index);
     }
@@ -859,6 +913,8 @@ void ExtraOptionDelegate::setModelData(QWidget *editor, QAbstractItemModel *mode
             model->setData(index, static_cast<QComboBox*>(editor)->currentText());
             break;
         }
+    } else if (index.data().type() == QVariant::Double) {
+        model->setData(index, static_cast<QLineEdit*>(editor)->text().toDouble());
     } else {
         QStyledItemDelegate::setModelData(editor, model, index);
     }
@@ -892,3 +948,17 @@ void ConfigWindow::on_reset_pushButton_clicked()
     updateGUI(true);
     populateComboBox();
 }
+
+ConfigWindow::StashItem::StashItem(SolverConfiguration* sc) :
+    json(sc->toJSONObject()),
+    isBuiltIn(sc->isBuiltin),
+    paramFile(sc->paramFile)
+{}
+
+SolverConfiguration* ConfigWindow::StashItem::consume() {
+    QStringList warnings;
+    auto* sc = new (SolverConfiguration) (SolverConfiguration::loadJSON(QJsonDocument(json), warnings));
+    sc->isBuiltin = isBuiltIn;
+    sc->paramFile = paramFile;
+    return sc;
+}
