@@ -9,12 +9,18 @@
 #include <QFileInfo>
 #include <QFileDialog>
 #include <QPushButton>
+#include <QRegularExpression>
 
 #include "ide.h"
+#include "ideutils.h"
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
 #include "checkupdatedialog.h"
 
+#ifdef MINIZINC_IDE_TESTING
+#include "tests/testide.h"
+#endif
+
 IDEStatistics::IDEStatistics(void)
     : errorsShown(0), errorsClicked(0), modelsRun(0) {}
 
@@ -26,9 +32,6 @@ void IDEStatistics::init(QVariant v) {
         modelsRun = m["modelsRun"].toInt();
         solvers = m["solvers"].toStringList();
     }
-#ifdef Q_OS_MAC
-    (void) new MyRtfMime();
-#endif
 }
 
 QVariantMap IDEStatistics::toVariantMap(void) {
@@ -130,7 +133,7 @@ void IDE::checkUpdate(void) {
             // Check if an update is available
             QNetworkRequest request(QUrl("http://www.minizinc.org/version-info.php"));
             versionCheckReply = networkManager->get(request);
-            connect(versionCheckReply, SIGNAL(finished()), this, SLOT(versionCheckFinished()));
+            connect(versionCheckReply, &QNetworkReply::finished, this, &IDE::versionCheckFinished);
         }
         QTimer::singleShot(24*60*60*1000, this, SLOT(checkUpdate()));
     }
@@ -142,10 +145,13 @@ IDE::IDE(int& argc, char* argv[]) : QApplication(argc,argv) {
     setApplicationVersion(MINIZINC_IDE_VERSION);
     setOrganizationName("MiniZinc");
     setOrganizationDomain("minizinc.org");
-#ifndef MINIZINC_IDE_BUNDLED
-    setApplicationName("MiniZinc IDE");
-#else
+#ifdef MINIZINC_IDE_TESTING
+    setApplicationName("MiniZinc IDE Test Suite");
+    TestMocker::resetSettings();
+#elif MINIZINC_IDE_BUNDLED
     setApplicationName("MiniZinc IDE (bundled)");
+#else
+    setApplicationName("MiniZinc IDE");
 #endif
 
     networkManager = new QNetworkAccessManager(this);
@@ -165,6 +171,7 @@ IDE::IDE(int& argc, char* argv[]) : QApplication(argc,argv) {
         settings.setValue("checkforupdates21",result==QDialog::Accepted);
         settings.sync();
     }
+    bool wordWrap = settings.value("wordWrap", true).toBool();
     settings.endGroup();
     settings.beginGroup("Recent");
     recentFiles = settings.value("files",QStringList()).toStringList();
@@ -175,6 +182,8 @@ IDE::IDE(int& argc, char* argv[]) : QApplication(argc,argv) {
 
     lastDefaultProject = nullptr;
 
+    darkModeNotifier = new DarkModeNotifier(this);
+
     { // Load cheat sheet
         QString fileContents;
         QFile file(":/cheat_sheet.mzn");
@@ -197,14 +206,18 @@ IDE::IDE(int& argc, char* argv[]) : QApplication(argc,argv) {
         }
         defaultFont.setStyleHint(QFont::TypeWriter);
         defaultFont.setPointSize(13);
-        QFont editorFont;
-        editorFont.fromString(settings.value("editorFont", defaultFont.toString()).value<QString>());
-        bool darkMode = settings.value("darkMode", false).value<bool>();
+        auto editorFont = IDEUtils::fontFromString(settings.value("editorFont").toString());
+        int zoom = settings.value("zoom", 100).toInt();
+        editorFont.setPointSize(editorFont.pointSize() * zoom / 100);
+        bool darkMode = darkModeNotifier->darkMode();
         settings.endGroup();
 
         cheatSheet = new QMainWindow;
         cheatSheet->setWindowTitle("MiniZinc Cheat Sheet");
-        CodeEditor* ce = new CodeEditor(nullptr,":/cheat_sheet.mzn",false,false,editorFont,darkMode,nullptr,nullptr);
+        CodeEditor* ce = new CodeEditor(nullptr,":/cheat_sheet.mzn",false,false,editorFont,2,false,darkMode,nullptr,nullptr);
+        ce->setWordWrapMode(wordWrap ?
+                                QTextOption::WrapAtWordBoundaryOrAnywhere :
+                                QTextOption::NoWrap);
         ce->document()->setPlainText(fileContents);
         QTextCursor cursor = ce->textCursor();
         cursor.movePosition(QTextCursor::Start);
@@ -215,10 +228,12 @@ IDE::IDE(int& argc, char* argv[]) : QApplication(argc,argv) {
         cheatSheet->resize(800, 600);
     }
 
+    connect(darkModeNotifier, &DarkModeNotifier::darkModeChanged, this, &IDE::onDarkModeChanged);
+    onDarkModeChanged(darkModeNotifier->darkMode());
 
-    connect(&fsWatch, SIGNAL(fileChanged(QString)), this, SLOT(fileModified(QString)));
-    connect(this, SIGNAL(focusChanged(QWidget*,QWidget*)), this, SLOT(handleFocusChange(QWidget*,QWidget*)));
-    connect(&modifiedTimer, SIGNAL(timeout()), this, SLOT(fileModifiedTimeout()));
+    connect(&fsWatch, &QFileSystemWatcher::fileChanged, this, &IDE::fileModified);
+    connect(this, &IDE::focusChanged, this, &IDE::handleFocusChange);
+    connect(&modifiedTimer, &QTimer::timeout, this, &IDE::fileModifiedTimeout);
 
 #ifdef Q_OS_MAC
     MainWindow* mw = new MainWindow(QString());
@@ -226,8 +241,8 @@ IDE::IDE(int& argc, char* argv[]) : QApplication(argc,argv) {
     defaultMenuBar = new QMenuBar(0);
     recentFilesMenu = new QMenu("Recent files");
     recentProjectsMenu = new QMenu("Recent projects");
-    connect(recentFilesMenu, SIGNAL(triggered(QAction*)), this, SLOT(recentFileMenuAction(QAction*)));
-    connect(recentProjectsMenu, SIGNAL(triggered(QAction*)), this, SLOT(recentProjectMenuAction(QAction*)));
+    connect(recentFilesMenu, &QMenu::triggered, this, &IDE::recentFileMenuAction);
+    connect(recentProjectsMenu, &QMenu::triggered, this, &IDE::recentProjectMenuAction);
     addRecentFile("");
     addRecentProject("");
 
@@ -248,13 +263,13 @@ IDE::IDE(int& argc, char* argv[]) : QApplication(argc,argv) {
                             QAction* na = nm->addAction(a->text());
                             na->setShortcut(a->shortcut());
                             if (a==mw->ui->actionQuit) {
-                                connect(na,SIGNAL(triggered()),this,SLOT(quit()));
+                                connect(na, &QAction::triggered, this, &IDE::quit);
                             } else if (a==mw->ui->actionNewModel_file || a==mw->ui->actionNew_project) {
-                                connect(na,SIGNAL(triggered()),this,SLOT(newProject()));
+                                connect(na, &QAction::triggered, this, &IDE::newProject);
                             } else if (a==mw->ui->actionOpen) {
-                                connect(na,SIGNAL(triggered()),this,SLOT(openFile()));
+                                connect(na, &QAction::triggered, this, [=] () { openFile(); });
                             } else if (a==mw->ui->actionHelp) {
-                                connect(na,SIGNAL(triggered()),this,SLOT(help()));
+                                connect(na, &QAction::triggered, this, &IDE::help);
                             } else {
                                 na->setEnabled(false);
                             }
@@ -426,28 +441,40 @@ void IDE::setLastPath(const QString& path) {
 
 void IDE::setEditorFont(QFont font)
 {
-    QSettings settings;
-    settings.beginGroup("MainWindow");
-    settings.setValue("editorFont", font.toString());
-    settings.endGroup();
-    for (QSet<MainWindow*>::iterator it = IDE::instance()->mainWindows.begin();
-         it != IDE::instance()->mainWindows.end(); ++it) {
-        (*it)->setEditorFont(font);
+    for (auto* mw : qAsConst(mainWindows)) {
+        mw->setEditorFont(font);
     }
+    static_cast<CodeEditor*>(cheatSheet->centralWidget())->setEditorFont(font);
+}
+
+void IDE::setEditorIndent(int indentSize, bool useTabs)
+{
+    for (auto* mw : qAsConst(mainWindows)) {
+        mw->setEditorIndent(indentSize, useTabs);
+    }
+}
+
+void IDE::setEditorWordWrap(QTextOption::WrapMode mode)
+{
+    for (auto* mw : qAsConst(mainWindows)) {
+        mw->setEditorWordWrap(mode);
+    }
+    static_cast<CodeEditor*>(cheatSheet->centralWidget())->setWordWrapMode(mode);
 }
 
 void IDE::openFile(const QString& fileName0)
 {
-    QString fileName = fileName0;
-    if (fileName.isEmpty()) {
-        fileName = QFileDialog::getOpenFileName(nullptr, tr("Open File"), getLastPath(), "MiniZinc Files (*.mzn *.dzn *.fzn *.json *.mzp *.mzc *.mpc);;Other (*)");
-        if (!fileName.isNull()) {
-            setLastPath(QFileInfo(fileName).absolutePath()+fileDialogSuffix);
+    QStringList fileNames;
+    if (fileName0.isEmpty()) {
+        fileNames = QFileDialog::getOpenFileNames(nullptr, tr("Open File"), getLastPath(), "MiniZinc Files (*.mzn *.dzn *.fzn *.json *.mzp *.mzc *.mpc);;Other (*)");
+        if (!fileNames.isEmpty()) {
+            setLastPath(QFileInfo(fileNames.last()).absolutePath() + fileDialogSuffix);
         }
+    } else {
+        fileNames << fileName0;
     }
-    if (!fileName.isEmpty()) {
-        MainWindow* mw = new MainWindow(QString());
-        mw->openFile(fileName, false, true);
+    if (!fileNames.isEmpty()) {
+        MainWindow* mw = new MainWindow(fileNames);
         mw->show();
     }
 }
@@ -523,7 +550,9 @@ void IDE::loadLargeFile(const QString &path, QWidget* parent)
         QFile file(path);
         if (file.open(QFile::ReadOnly | QFile::Text)) {
             QTextStream file_stream(&file);
+#if QT_VERSION < 0x060000
             file_stream.setCodec("UTF-8");
+#endif
             it.value()->td.setPlainText(file_stream.readAll());
             it.value()->large = false;
             it.value()->td.setModified(false);
@@ -582,29 +611,31 @@ IDE::versionCheckFinished(void) {
     if (versionCheckReply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt()==200) {
         QString currentVersion = versionCheckReply->readAll();
 
-        QRegExp versionRegExp("([1-9][0-9]*)\\.([0-9]+)\\.([0-9]+)");
+        QRegularExpression versionRegExp("([1-9][0-9]*)\\.([0-9]+)\\.([0-9]+)");
 
         int curVersionMajor = 0;
         int curVersionMinor = 0;
         int curVersionPatch = 0;
         bool ok = true;
-        if (versionRegExp.indexIn(currentVersion) != -1) {
-            curVersionMajor = versionRegExp.cap(1).toInt(&ok);
+        QRegularExpressionMatch curVersionMatch = versionRegExp.match(currentVersion);
+        if (curVersionMatch.hasMatch()) {
+            curVersionMajor = curVersionMatch.captured(1).toInt(&ok);
             if (ok)
-                curVersionMinor = versionRegExp.cap(2).toInt(&ok);
+                curVersionMinor = curVersionMatch.captured(2).toInt(&ok);
             if (ok)
-                curVersionPatch = versionRegExp.cap(3).toInt(&ok);
+                curVersionPatch = curVersionMatch.captured(3).toInt(&ok);
         }
 
         int appVersionMajor = 0;
         int appVersionMinor = 0;
         int appVersionPatch = 0;
-        if (ok && versionRegExp.indexIn(applicationVersion()) != -1) {
-            appVersionMajor = versionRegExp.cap(1).toInt(&ok);
+        QRegularExpressionMatch appVersionMatch = versionRegExp.match(applicationVersion());
+        if (ok && appVersionMatch.hasMatch()) {
+            appVersionMajor = appVersionMatch.captured(1).toInt(&ok);
             if (ok)
-                appVersionMinor = versionRegExp.cap(2).toInt(&ok);
+                appVersionMinor = appVersionMatch.captured(2).toInt(&ok);
             if (ok)
-                appVersionPatch = versionRegExp.cap(3).toInt(&ok);
+                appVersionPatch = appVersionMatch.captured(3).toInt(&ok);
         }
 
         bool needUpdate = ok && (curVersionMajor > appVersionMajor ||
@@ -642,3 +673,23 @@ QString IDE::appDir(void) const {
 IDE* IDE::instance(void) {
     return static_cast<IDE*>(qApp);
 }
+
+void IDE::onDarkModeChanged(bool darkMode)
+{
+    static_cast<CodeEditor*>(cheatSheet->centralWidget())->setDarkMode(darkMode);
+
+    if (!darkModeNotifier->hasNativeDarkMode()) {
+        // No native dark widgets, so use stylesheet instead
+        if (darkMode) {
+            QFile sheet(":/dark_mode.css");
+            sheet.open(QFile::ReadOnly);
+            qApp->setStyleSheet(sheet.readAll());
+        } else {
+            qApp->setStyleSheet("");
+        }
+    }
+
+    for (auto* mw : qAsConst(mainWindows)) {
+        mw->setDarkMode(darkMode);
+    }
+}
