@@ -18,11 +18,6 @@
 #include <QProcess>
 #include <QTimer>
 #include <QLabel>
-#ifdef MINIZINC_IDE_HAVE_WEBENGINE
-#include <QWebEngineView>
-#else
-#include <QWebView>
-#endif
 #include <QSet>
 #include <QTemporaryDir>
 #include <QMap>
@@ -32,16 +27,18 @@
 #include <QProgressBar>
 #include <QComboBox>
 #include <QVersionNumber>
+#include <QJsonArray>
 
 #include "codeeditor.h"
-#include "solverdialog.h"
+#include "solver.h"
 #include "paramdialog.h"
 #include "project.h"
-#include "htmlwindow.h"
 #include "moocsubmission.h"
-#include "solverconfiguration.h"
+#include "solver.h"
 #include "process.h"
 #include "codechecker.h"
+#include "profilecompilation.h"
+#include "server.h"
 
 #include "../cp-profiler/src/cpprofiler/conductor.hh"
 #include "../cp-profiler/src/cpprofiler/execution_window.hh"
@@ -60,6 +57,8 @@ class MainWindow : public QMainWindow
 {
     Q_OBJECT
     friend class IDE;
+    friend class TestIDE;
+
 public:
     explicit MainWindow(const QString& project = QString());
     explicit MainWindow(const QStringList& files);
@@ -83,11 +82,16 @@ private:
 signals:
     /// emitted when compilation and running of a model has finished
     void finished();
+    /// emitted when the window is going to close
+    void terminating();
 
 public slots:
 
     void openFile(const QString &path = QString(), bool openAsModified=false, bool focus=true);
     void stop();
+    void setDarkMode(bool enable);
+    void initTheme();
+    void closeFindWidget();
 
 private slots:
 
@@ -113,7 +117,7 @@ private slots:
 
     void openCompiledFzn(const QString& file);
 
-    void profileCompiledFzn(const QString& output);
+    void profileCompiledFzn(const QVector<TimingEntry>& timing, const QVector<PathEntry>& paths);
 
     void on_actionSave_as_triggered();
 
@@ -125,7 +129,7 @@ private slots:
 
     void on_actionAbout_MiniZinc_IDE_triggered();
 
-    void errorClicked(const QUrl&);
+    void anchorClicked(const QUrl&);
 
     void on_actionDefault_font_size_triggered();
 
@@ -135,8 +139,6 @@ private slots:
 
     void on_actionReplace_triggered();
 
-    void on_actionSelect_font_triggered();
-
     void on_actionGo_to_line_triggered();
 
     void on_actionShift_left_triggered();
@@ -197,17 +199,9 @@ private slots:
 
     void showWindowMenu(void);
     void windowMenuSelected(QAction*);
-    void closeHTMLWindow(int);
     void on_actionCheat_Sheet_triggered();
     void check_code();
 
-    void on_actionDark_mode_toggled(bool arg1);
-
-    //Themes
-    void on_actionMangoTheme_triggered();
-    void on_actionBlueberryTheme_triggered();
-    void on_actionDefaultTheme_triggered();
-
     void moocFinished(int);
 
     void on_actionEditSolverConfig_triggered();
@@ -253,48 +247,38 @@ private slots:
 
     void on_menuSolver_configurations_triggered(QAction* action);
 
-    void on_solutionOutput(const QString& solution);
-    void on_finalStatus(const QString& status);
-    void on_fragment(const QString& data);
     void on_progressOutput(float progress);
 
+    void on_minizincError(const QJsonObject& error);
+
 protected:
     virtual void closeEvent(QCloseEvent*);
     virtual void dragEnterEvent(QDragEnterEvent *);
     virtual void dropEvent(QDropEvent *);
-#ifdef Q_OS_MAC
-    virtual void paintEvent(QPaintEvent *);
-#endif
     bool eventFilter(QObject *, QEvent *);
-    void compileAndRun(const QString& modelPath, const QString& additionalCmdlineParams, const QStringList& additionalDataFiles, const QStringList& additionalMznParams);
 public:
-    void resolve(int htmlWindowIdentifier, const QString &data);
     QString currentSolver(void) const;
     QString currentSolverConfigName(void);
-    int addHtmlWindow(HTMLWindow* w);
     bool checkSolutions(void) const;
     void setCheckSolutions(bool b);
     SolverConfiguration* getCurrentSolverConfig(void);
     const Solver* getCurrentSolver(void);
+    void compileSolutionChecker(const QString& checker);
     void compile(const SolverConfiguration& sc, const QString& model, const QStringList& data = QStringList(), const QStringList& extraArgs = QStringList(), bool profile = false);
     void run(const SolverConfiguration& sc, const QString& model, const QStringList& data = QStringList(), const QStringList& extraArgs = QStringList(), QTextStream* ts = nullptr);
+    QList<CodeEditor*> codeEditors();
+    bool isDarkMode() const { return darkMode; }
 private:
     Ui::MainWindow *ui;
     CodeEditor* curEditor;
-    QString curFilePath;
-    int curHtmlWindow;
-    QVector<HTMLWindow*> htmlWindows;
-    QVector<QString> htmlWindowModels;
     CodeChecker* code_checker;
     CodeEditor* curCheckEditor;
-    int solutionCount;
-    int solutionLimit;
-    QTimer solutionLimitTimer;
-    QVector<QString> hiddenSolutions;
     QProgressBar* progressBar;
     QLabel* statusLabel;
     QLabel* statusLineColLabel;
     QFont editorFont;
+    int indentSize;
+    bool useTabs;
     bool darkMode;
     QVector<Solver> solvers;
     QStringList currentAdditionalDataFiles;
@@ -305,7 +289,6 @@ private:
     bool saveBeforeRunning;
     ParamDialog* paramDialog;
     bool profileInfoVisible;
-    int runTimeout;
     Project* project;
     int newFileCounter;
     QComboBox* solverConfCombo;
@@ -313,17 +296,14 @@ private:
     QAction* fakeStopAction;
     QAction* fakeCompileAction;
     QAction* minimizeAction;
-    QTextStream* outputBuffer;
     MOOCSubmission* moocSubmission;
     bool processRunning;
 
     QToolButton* runButton;
-    QVector<QPair<SolverFlag,QWidget*>> extraSolverFlags;
-    int currentSolverConfig;
-    bool renamingSolverConf;
 
     cpprofiler::Conductor* conductor = nullptr;
-    int ex_id;
+    Server* server = nullptr;
+    VisConnector* vis_connector = nullptr;
 
     void createEditor(const QString& path, bool openAsModified, bool isNewFile, bool readOnly=false, bool focus=true);
     enum ConfMode { CONF_CHECKARGS, CONF_COMPILE, CONF_RUN };
@@ -332,6 +312,8 @@ private:
     void saveProject(const QString& filepath);
     void loadProject(const QString& filepath);
     void setEditorFont(QFont font);
+    void setEditorIndent(int indentSize, bool useTabs);
+    void setEditorWordWrap(QTextOption::WrapMode mode);
     void setLastPath(const QString& s);
     QString getLastPath(void);
     QString setElapsedTime(qint64 elapsed_t);
@@ -346,17 +328,15 @@ private:
     void outputStdErr(const QString& line);
     QStringList getOpenFiles(void);
 
-    void openJSONViewer(bool isJSONinitHandler, const QVector<SolveProcess::VisOutput>& output);
-    void finishJSONViewer(qint64 time);
-
     void updateProfileSearchButton(void);
 
-    QString runMessage(const QString& action, const QString& model, const QStringList& data, const QStringList& extraArgs);
+    QString locationToLink(const QString& filename, int firstLine, int firstColumn, int lastLine, int lastColumn, const QColor& color);
+
+    void startVisualisation(const QString& model, const QStringList& data, const QString& url, const QJsonValue& userData, MznProcess* proc);
 public:
     void addOutput(const QString& s, bool html=true);
     void openProject(const QString& fileName);
     bool isEmptyProject(void);
-    void selectJSONSolution(HTMLPage* source, int n);
     Project& getProject(void) { return *project; }
 };
 
