@@ -46,6 +46,8 @@ void MOOCAssignment::loadJSON(const QJsonObject& obj, const QFileInfo& fi)
     moocName = obj["moocName"].toString();
     moocPasswordString = obj["moocPasswordString"].toString();
     submissionURL = obj["submissionURL"].toString();
+    submissionTerms = obj["submissionTerms"].toString();
+    sendMeta = obj["sendMeta"].toBool(false);
     QJsonArray sols = obj["solutionAssignments"].toArray();
     for (int i=0; i<sols.size(); i++) {
         QJsonObject solO = sols[i].toObject();
@@ -58,14 +60,14 @@ void MOOCAssignment::loadJSON(const QJsonObject& obj, const QFileInfo& fi)
         QFileInfo model_fi(fi.dir(), solO["model"].toString());
         QFileInfo data_fi(fi.dir(), solO["data"].toString());
         MOOCAssignmentItem item(solO["id"].toString(), model_fi.absoluteFilePath(), data_fi.absoluteFilePath(),
-                                timeout, solO["name"].toString());
+                                timeout, solO["name"].toString(), solO["required"].toBool(false));
         problems.append(item);
     }
     QJsonArray modelsArray = obj["modelAssignments"].toArray();
     for (int i=0; i<modelsArray.size(); i++) {
         QJsonObject modelO = modelsArray[i].toObject();
         QFileInfo model_fi(fi.dir(), modelO["model"].toString());
-        MOOCAssignmentItem item(modelO["id"].toString(), model_fi.absoluteFilePath(), modelO["name"].toString());
+        MOOCAssignmentItem item(modelO["id"].toString(), model_fi.absoluteFilePath(), modelO["name"].toString(), modelO["required"].toBool(false));
         models.append(item);
     }
 }
@@ -89,16 +91,48 @@ MOOCSubmission::MOOCSubmission(MainWindow* mw0, MOOCAssignment& cp) :
     QVBoxLayout* problemLayout = new QVBoxLayout;
     ui->problemBox->setLayout(problemLayout);
 
+    QSet<QString> mooc_mzns;
+    for (auto& p : project.models) {
+        mooc_mzns << p.model;
+    }
+    for (auto& p : project.problems) {
+        mooc_mzns << p.model;
+    }
+    QStringList nonSubmissionFiles;
+    for (auto* ce : mw->codeEditors()) {
+        if (ce->filename.endsWith(".mzn") && !mooc_mzns.contains(ce->filepath)) {
+            nonSubmissionFiles << ce->filename;
+        }
+    }
+    if (nonSubmissionFiles.empty()) {
+        ui->nonSubmissionGroup->setVisible(false);
+    } else {
+        ui->nonSubmissionLabel->setText(nonSubmissionFiles.join("\n"));
+        ui->nonSubmissionGroup->setVisible(true);
+    }
+
+    if (project.submissionTerms.isEmpty()) {
+        ui->submissionTerms_groupBox->setVisible(false);
+        ui->runButton->setEnabled(true);
+    } else {
+        ui->submissionTerms_textBrowser->setText(cp.submissionTerms);
+        ui->submissionTerms_checkBox->setChecked(false);
+        ui->runButton->setEnabled(false);
+        ui->submissionTerms_groupBox->setVisible(true);
+    }
+
     for (int i=0; i<project.models.size(); i++) {
         const MOOCAssignmentItem& item = project.models.at(i);
         QCheckBox* cb = new QCheckBox(item.name);
         cb->setChecked(true);
+        cb->setEnabled(!item.required);
         modelLayout->addWidget(cb);
     }
     for (int i=0; i<project.problems.size(); i++) {
         const MOOCAssignmentItem& item = project.problems.at(i);
         QCheckBox* cb = new QCheckBox(item.name);
         cb->setChecked(true);
+        cb->setEnabled(!item.required);
         problemLayout->addWidget(cb);
     }
     if (project.models.empty())
@@ -139,6 +173,7 @@ void MOOCSubmission::disableUI()
     ui->loginGroup->setEnabled(false);
     ui->modelBox->setEnabled(false);
     ui->problemBox->setEnabled(false);
+    ui->submissionTerms_checkBox->setEnabled(false);
     ui->runButton->setText("Abort");
 }
 
@@ -147,6 +182,7 @@ void MOOCSubmission::enableUI()
     ui->loginGroup->setEnabled(true);
     ui->modelBox->setEnabled(true);
     ui->problemBox->setEnabled(true);
+    ui->submissionTerms_checkBox->setEnabled(true);
     ui->runButton->setText("Run and submit");
 }
 
@@ -156,14 +192,14 @@ void MOOCSubmission::cancelOperation()
     case S_NONE:
         return;
     case S_WAIT_PWD:
-        disconnect(mw, SIGNAL(finished()), this, SLOT(rcvLoginCheckResponse()));
+        disconnect(mw, &MainWindow::finished, this, &MOOCSubmission::rcvLoginCheckResponse);
         break;
     case S_WAIT_SOLVE:
-        disconnect(mw, SIGNAL(finished()), this, SLOT(solverFinished()));
+        disconnect(mw, &MainWindow::finished, this, &MOOCSubmission::solverFinished);
         mw->stop();
         break;
     case S_WAIT_SUBMIT:
-        disconnect(reply, SIGNAL(finished()), this, SLOT(rcvSubmissionResponse()));
+        disconnect(reply, &QNetworkReply::finished, this, &MOOCSubmission::rcvSubmissionResponse);
         break;
     }
     ui->textBrowser->insertPlainText("Aborted.\n");
@@ -200,7 +236,7 @@ void MOOCSubmission::solveNext() {
     if (_current_model < n_problems) {
 
         MOOCAssignmentItem& item = project.problems[_current_model];
-        connect(mw, SIGNAL(finished()), this, SLOT(solverFinished()));
+        connect(mw, &MainWindow::finished, this, &MOOCSubmission::solverFinished);
         ui->textBrowser->insertPlainText("Running "+item.name+" with time out " + QString().setNum(item.timeout)+ "s\n");
         _cur_phase = S_WAIT_SOLVE;
         _output_string = "";
@@ -261,20 +297,41 @@ void MOOCSubmission::submitToMOOC()
     _submission["secret"] = ui->password->text();
     _submission["submitterEmail"] = ui->login->text();
     _submission["parts"] = _parts;
+    if (project.sendMeta) {
+        QJsonObject metadata;
+        metadata["version"] = MznDriver::get().version().toString();
+        metadata["versionString"] = MznDriver::get().minizincVersionString();
+        metadata["solver"] = mw->getCurrentSolver()->id + " " + mw->getCurrentSolver()->version;
+        metadata["arch"] = QSysInfo::currentCpuArchitecture();
+        metadata["kernelType"] = QSysInfo::kernelType();
+        metadata["kernelVersion"] = QSysInfo::kernelVersion();
+        metadata["productType"] = QSysInfo::productType();
+        metadata["productVersion"] = QSysInfo::productVersion();
+        metadata["prettyProductName"] = QSysInfo::prettyProductName();
+        _submission["metadata"] = metadata;
+    }
 
     QJsonDocument doc(_submission);
 
     _cur_phase = S_WAIT_SUBMIT;
     reply = IDE::instance()->networkManager->post(request,doc.toJson());
-    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(rcvErrorResponse(QNetworkReply::NetworkError)));
-    connect(reply, SIGNAL(finished()), this, SLOT(rcvSubmissionResponse()));
+#if QT_VERSION >= 0x060000
+    connect(reply, &QNetworkReply::errorOccurred, this, &MOOCSubmission::rcvErrorResponse);
+#else
+    connect(reply, QOverload<QNetworkReply::NetworkError>::of(&QNetworkReply::error), this, &MOOCSubmission::rcvErrorResponse);
+#endif
+    connect(reply, &QNetworkReply::finished, this, &MOOCSubmission::rcvSubmissionResponse);
     ui->textBrowser->insertPlainText("Submitting to "+project.moocName+" for grading...\n");
 }
 
 void MOOCSubmission::rcvSubmissionResponse()
 {
-    disconnect(reply, SIGNAL(finished()), this, SLOT(rcvSubmissionResponse()));
-    disconnect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(rcvErrorResponse(QNetworkReply::NetworkError)));
+    disconnect(reply, &QNetworkReply::finished, this, &MOOCSubmission::rcvSubmissionResponse);
+#if QT_VERSION >= 0x060000
+    disconnect(reply, &QNetworkReply::errorOccurred, this, &MOOCSubmission::rcvErrorResponse);
+#else
+    disconnect(reply, QOverload<QNetworkReply::NetworkError>::of(&QNetworkReply::error), this, &MOOCSubmission::rcvErrorResponse);
+#endif
     reply->deleteLater();
 
     QJsonDocument doc = QJsonDocument::fromJson(reply->readAll());
@@ -297,7 +354,7 @@ void MOOCSubmission::rcvSubmissionResponse()
 
 void MOOCSubmission::solverFinished()
 {
-    disconnect(mw, SIGNAL(finished()), this, SLOT(solverFinished()));
+    disconnect(mw, &MainWindow::finished, this, &MOOCSubmission::solverFinished);
 
     QStringList solutions = _output_string.split("----------");
     if (solutions.size() >= 2) {
@@ -347,8 +404,12 @@ void MOOCSubmission::on_runButton_clicked()
         _cur_phase = S_WAIT_PWD;
         disableUI();
         reply = IDE::instance()->networkManager->post(request,doc.toJson());
-        connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(rcvErrorResponse(QNetworkReply::NetworkError)));
-        connect(reply, SIGNAL(finished()), this, SLOT(rcvLoginCheckResponse()));
+#if QT_VERSION >= 0x060000
+        connect(reply, &QNetworkReply::errorOccurred, this, &MOOCSubmission::rcvErrorResponse);
+#else
+        connect(reply, QOverload<QNetworkReply::NetworkError>::of(&QNetworkReply::error), this, &MOOCSubmission::rcvErrorResponse);
+#endif
+        connect(reply, &QNetworkReply::finished, this, &MOOCSubmission::rcvLoginCheckResponse);
         ui->textBrowser->insertPlainText("Checking login and assignment token...\n");
     } else {
         cancelOperation();
@@ -357,8 +418,12 @@ void MOOCSubmission::on_runButton_clicked()
 
 void MOOCSubmission::rcvLoginCheckResponse()
 {
-    disconnect(reply, SIGNAL(finished()), this, SLOT(rcvLoginCheckResponse()));
-    disconnect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(rcvErrorResponse(QNetworkReply::NetworkError)));
+    disconnect(reply, &QNetworkReply::finished, this, &MOOCSubmission::rcvLoginCheckResponse);
+#if QT_VERSION >= 0x060000
+    disconnect(reply, &QNetworkReply::errorOccurred, this, &MOOCSubmission::rcvErrorResponse);
+#else
+    disconnect(reply, QOverload<QNetworkReply::NetworkError>::of(&QNetworkReply::error), this, &MOOCSubmission::rcvErrorResponse);
+#endif
     reply->deleteLater();
 
     QJsonDocument doc = QJsonDocument::fromJson(reply->readAll());
@@ -388,10 +453,20 @@ void MOOCSubmission::rcvLoginCheckResponse()
     }
 }
 
-void MOOCSubmission::rcvErrorResponse(QNetworkReply::NetworkError)
+void MOOCSubmission::rcvErrorResponse(QNetworkReply::NetworkError e)
 {
-    if (!reply->errorString().endsWith("Bad Request")) {
-        ui->textBrowser->insertPlainText("Error:\n"+reply->errorString()+"\n\n");
+    switch (e) {
+    case QNetworkReply::ProtocolInvalidOperationError:
+    case QNetworkReply::AuthenticationRequiredError:
+        // ProtocolInvalidOperationError expected when checking auth succeeds
+        // AuthenticationRequiredError expected when checking auth fails
+        break;
+    default:
+        // Unknown networking error
+        // TODO: can probably remove this check since it should be covered by the ProtocolInvalidOperationError
+        if (!reply->errorString().endsWith("Bad Request")) {
+            ui->textBrowser->insertPlainText("Error:\n" + reply->errorString() + "\n\n");
+        }
     }
 }
 
@@ -405,3 +480,8 @@ void MOOCSubmission::on_storePassword_toggled(bool checked)
         settings.endGroup();
     }
 }
+
+void MOOCSubmission::on_submissionTerms_checkBox_stateChanged(int arg1)
+{
+    ui->runButton->setEnabled(arg1);
+}
