@@ -19,6 +19,12 @@
 #include <unistd.h>
 #include <signal.h>
 #include <iostream>
+#else
+#include <VersionHelpers.h>
+#endif
+
+#ifdef Q_OS_MAC
+#include <sys/sysctl.h>
 #endif
 
 #ifdef Q_OS_WIN
@@ -37,15 +43,25 @@ void Process::start(const QString &program, const QStringList &arguments, const
     env.insert("PATH", addPath + pathSep + curPath);
     setProcessEnvironment(env);
 #ifdef Q_OS_WIN
-    _putenv_s("PATH", (addPath + pathSep + curPath).toStdString().c_str());
-    jobObject = CreateJobObject(nullptr, nullptr);
-    connect(this, SIGNAL(started()), this, SLOT(attachJob()));
+    _wputenv_s(L"PATH", (addPath + pathSep + curPath).toStdWString().c_str());
+    if (IsWindows8OrGreater()) {
+        jobObject = CreateJobObject(nullptr, nullptr);
+        connect(this, &QProcess::started, this, &Process::attachJob);
+    } else {
+        // Workaround PCA automatically adding to a job for Windows 7
+        setCreateProcessArgumentsModifier([] (QProcess::CreateProcessArguments *args) {
+            args->flags |= CREATE_BREAKAWAY_FROM_JOB;
+        });
+    }
 #else
+#if QT_VERSION >= 0x060000
+    setChildProcessModifier(Process::setpgid);
+#endif
     setenv("PATH", (addPath + pathSep + curPath).toStdString().c_str(), 1);
 #endif
     QProcess::start(program,arguments, QIODevice::Unbuffered | QIODevice::ReadWrite);
 #ifdef Q_OS_WIN
-    _putenv_s("PATH", curPath.toStdString().c_str());
+    _wputenv_s(L"PATH", curPath.toStdWString().c_str());
 #else
     setenv("PATH", curPath.toStdString().c_str(), 1);
 #endif
@@ -55,7 +71,12 @@ void Process::terminate()
 {
     if (state() != QProcess::NotRunning) {
 #ifdef Q_OS_WIN
-        TerminateJobObject(jobObject, EXIT_FAILURE);
+        if (IsWindows8OrGreater()) {
+            TerminateJobObject(jobObject, EXIT_FAILURE);
+        } else {
+            // We can't use job objects in Windows 7, since MiniZinc already uses them
+            QProcess::kill();
+        }
 #else
         ::killpg(processId(), SIGKILL);
 #endif
@@ -78,7 +99,8 @@ void Process::sendInterrupt()
     auto pipeName = pipe.toStdString();
     HANDLE hNamedPipe = CreateFileA(pipeName.c_str(), GENERIC_WRITE, 0, nullptr, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, nullptr);
     if (hNamedPipe != INVALID_HANDLE_VALUE) {
-        WriteFile(hNamedPipe, nullptr, 0, nullptr, nullptr);
+        DWORD bytesWritten;
+        WriteFile(hNamedPipe, nullptr, 0, &bytesWritten, nullptr);
         CloseHandle(hNamedPipe);
     }
 #else
@@ -86,7 +108,11 @@ void Process::sendInterrupt()
 #endif
 }
 
+#if QT_VERSION >= 0x060000
+void Process::setpgid()
+#else
 void Process::setupChildProcess()
+#endif
 {
 #ifndef Q_OS_WIN
     if (::setpgid(0,0)) {
@@ -98,7 +124,10 @@ void Process::setupChildProcess()
 #ifdef Q_OS_WIN
 void Process::attachJob()
 {
-    AssignProcessToJobObject(jobObject, pid()->hProcess);
+    HANDLE hProcess = OpenProcess(PROCESS_ALL_ACCESS, true, processId());
+    if (hProcess != nullptr) {
+        AssignProcessToJobObject(jobObject, hProcess);
+    }
 }
 #endif
 
@@ -106,10 +135,42 @@ void MznDriver::setLocation(const QString &mznDistribPath)
 {
     clear();
 
-    _minizincExecutable = "minizinc";
     _mznDistribPath = mznDistribPath;
-
     MznProcess p;
+    QRegularExpression version_regexp("version (\\d+)\\.(\\d+)\\.(\\d+)");
+    _minizincExecutable = QStringList({"minizinc"});
+
+#ifdef Q_OS_MAC
+    int isTranslated = 0;
+    {
+        size_t size = sizeof(isTranslated);
+        if (sysctlbyname("sysctl.proc_translated", &isTranslated, &size, NULL, 0) == -1) {
+            isTranslated = 0;
+        }
+    }
+
+    if (isTranslated) {
+        _minizincExecutable = QStringList({"arch", "-arch", "arm64", "minizinc"});
+    }
+    for (int i = 0; i <= isTranslated; i++) {
+        if (i == 1) {
+            _minizincExecutable = QStringList({"minizinc"});
+        }
+        try {
+            auto result = p.run({"--version"});
+            _versionString = result.stdOut + result.stdErr;
+            QRegularExpressionMatch path_match = version_regexp.match(_versionString);
+            if (path_match.hasMatch()) {
+                break;
+            }
+        } catch (ProcessError& e) {
+            if (i == isTranslated) {
+                clear();
+                throw;
+            }
+        }
+    }
+#else
     try {
         auto result = p.run({"--version"});
         _versionString = result.stdOut + result.stdErr;
@@ -117,8 +178,8 @@ void MznDriver::setLocation(const QString &mznDistribPath)
         clear();
         throw;
     }
+#endif
 
-    QRegularExpression version_regexp("version (\\d+)\\.(\\d+)\\.(\\d+)");
     QRegularExpressionMatch path_match = version_regexp.match(_versionString);
     if (path_match.hasMatch()) {
         _version = QVersionNumber(
@@ -132,7 +193,7 @@ void MznDriver::setLocation(const QString &mznDistribPath)
         throw DriverError(message);
     }
 
-    auto minVersion = QVersionNumber(2, 5, 0);
+    auto minVersion = QVersionNumber(2, 6, 0);
     if (_version < minVersion) {
         clear();
         throw DriverError("Versions of MiniZinc before " + minVersion.toString() + " are not supported.");
@@ -150,21 +211,23 @@ void MznDriver::setLocation(const QString &mznDistribPath)
     allOutput = p.run({"--solvers-json"}).stdOut.toUtf8();
     jd = QJsonDocument::fromJson(allOutput.toUtf8());
     if (!jd.isNull()) {
+        for (auto* s : _solvers) {
+            delete s;
+        }
         _solvers.clear();
         QJsonArray allSolvers = jd.array();
         for (auto item : allSolvers) {
             QJsonObject sj = item.toObject();
-            Solver s(sj);
-            _solvers.append(s);
+            _solvers.append(new Solver(sj));
         }
     }
 }
 
 Solver* MznDriver::defaultSolver(void)
 {
-    for (auto& solver : solvers()) {
-        if (solver.isDefaultSolver) {
-            return &solver;
+    for (auto* solver : solvers()) {
+        if (solver->isDefaultSolver) {
+            return solver;
         }
     }
     return nullptr;
@@ -172,8 +235,8 @@ Solver* MznDriver::defaultSolver(void)
 
 void MznDriver::setDefaultSolver(const Solver& s)
 {
-    for (auto& solver : solvers()) {
-        solver.isDefaultSolver = &solver == &s;
+    for (auto* solver : solvers()) {
+        solver->isDefaultSolver = *solver == s;
     }
 
     QFile uc(userConfigFile());
@@ -219,6 +282,7 @@ void MznDriver::setDefaultSolver(const Solver& s)
         throw DriverError("Cannot write user configuration file " + userConfigFile());
     }
 }
+
 void MznProcess::start(const QStringList& args, const QString& cwd)
 {
     p.setWorkingDirectory(cwd);
@@ -228,18 +292,20 @@ void MznProcess::start(const QStringList& args, const QString& cwd)
         timer.start(200);
         emit started();
     });
-    connect(&p, QOverload<int, QProcess::ExitStatus>::of(&Process::finished), [=](int code, QProcess::ExitStatus e) {
+    connect(&p, QOverload<int, QProcess::ExitStatus>::of(&Process::finished), this, [=](int code, QProcess::ExitStatus e) {
         flushOutput();
-        if (code == 0 || ignoreExitStatus) {
-            emit success();
+        timer.stop();
+        if (code == 0 || cancelled) {
+            emit success(cancelled);
         } else {
             emit failure(code, FailureType::NonZeroExit);
         }
         p.disconnect();
         emit(finished(elapsedTime()));
     });
-    connect(&p, &QProcess::errorOccurred, [=](QProcess::ProcessError e) {
-        if (!ignoreExitStatus) {
+    connect(&p, &QProcess::errorOccurred, this, [=](QProcess::ProcessError e) {
+        timer.stop();
+        if (!cancelled) {
             flushOutput();
             switch (e) {
             case QProcess::FailedToStart:
@@ -260,11 +326,16 @@ void MznProcess::start(const QStringList& args, const QString& cwd)
     connect(&p, &QProcess::readyReadStandardError, this, &MznProcess::readStdErr);
 
     Q_ASSERT(p.state() == QProcess::NotRunning);
-    ignoreExitStatus = false;
-    p.start(MznDriver::get().minizincExecutable(), args, MznDriver::get().mznDistribPath());
+    cancelled = false;
+    const QStringList& exec = MznDriver::get().minizincExecutable();
+    QStringList execArgs = args;
+    for (unsigned int i = exec.size() - 1; i > 0; i--) {
+        execArgs.push_front(exec[i]);
+    }
+    p.start(exec[0], execArgs, MznDriver::get().mznDistribPath());
 }
 
-void MznProcess::start(const SolverConfiguration& sc, const QStringList& args, const QString& cwd)
+void MznProcess::start(const SolverConfiguration& sc, const QStringList& args, const QString& cwd, bool jsonStream)
 {
     auto* temp = new QTemporaryFile(QDir::tempPath() + "/mzn_XXXXXX.mpc", this);
     if (!temp->open()) {
@@ -278,11 +349,24 @@ void MznProcess::start(const SolverConfiguration& sc, const QStringList& args, c
         delete temp;
     });
     QStringList newArgs;
-    newArgs << "--param-file-no-push" << paramFile << args;
+    parse = jsonStream;
+    if (jsonStream) {
+        newArgs << "--json-stream";
+    }
+    if (!sc.paramFile.isEmpty()) {
+        QFileInfo fi(sc.paramFile);
+        newArgs << "--push-working-directory"
+                << fi.canonicalPath();
+    }
+    newArgs << "--param-file-no-push" << paramFile;
+    if (!sc.paramFile.isEmpty()) {
+        newArgs << "--pop-working-directory";
+    }
+    newArgs << args;
     if (sc.timeLimit != 0) {
         auto* hardTimer = new QTimer(this);
         hardTimer->setSingleShot(true);
-        connect(hardTimer, &QTimer::timeout, hardTimer, [=] () {
+        connect(hardTimer, &QTimer::timeout, this, [=] () {
             stop();
         });
         connect(this, &MznProcess::started, hardTimer, [=] () {
@@ -300,7 +384,12 @@ MznProcess::RunResult MznProcess::run(const QStringList& args, const QString& cw
 {
     Q_ASSERT(p.state() == QProcess::NotRunning);
     p.setWorkingDirectory(cwd);
-    p.start(MznDriver::get().minizincExecutable(), args, MznDriver::get().mznDistribPath());
+    const QStringList& exec = MznDriver::get().minizincExecutable();
+    QStringList execArgs = args;
+    for (unsigned int i = exec.size() - 1; i > 0; i--) {
+        execArgs.push_front(exec[i]);
+    }
+    p.start(exec[0], execArgs, MznDriver::get().mznDistribPath());
     if (!p.waitForStarted()) {
         throw ProcessError("Failed to find or start minizinc " + args.join(" ") + ".");
     }
@@ -331,16 +420,16 @@ void MznProcess::stop()
     if (p.state() == QProcess::NotRunning) {
         return;
     }
-    ignoreExitStatus = true;
+    cancelled = true;
     p.sendInterrupt();
     auto* killTimer = new QTimer(this);
     killTimer->setSingleShot(true);
-    connect(killTimer, &QTimer::timeout, killTimer, [=] () {
+    connect(killTimer, &QTimer::timeout, this, [=] () {
         if (p.state() == QProcess::Running) {
             terminate();
         }
     });
-    connect(this, &MznProcess::finished, [=] () {
+    connect(this, &MznProcess::finished, killTimer, [=] () {
         killTimer->stop();
         delete killTimer;
     });
@@ -355,9 +444,9 @@ void MznProcess::terminate()
 
     p.disconnect();
     p.terminate();
-    emit success();
-    emit finished(timer.elapsed());
     timer.stop();
+    emit success(cancelled);
+    emit finished(timer.elapsed());
 }
 
 qint64 MznProcess::elapsedTime()
@@ -375,6 +464,25 @@ void MznProcess::closeStdIn()
     p.closeWriteChannel();
 }
 
+QString MznProcess::command() const
+{
+    return p.program() + " " + p.arguments().join(" ");
+}
+
+void MznProcess::processOutput()
+{
+    p.setReadChannel(QProcess::ProcessChannel::StandardOutput);
+    while (p.canReadLine()) {
+        auto line = QString::fromUtf8(p.readLine());
+        onStdOutLine(line);
+    }
+    p.setReadChannel(QProcess::ProcessChannel::StandardError);
+    while (p.canReadLine()) {
+        auto line = QString::fromUtf8(p.readLine());
+        emit outputStdError(line);
+    }
+}
+
 void MznProcess::readStdOut()
 {
     p.setReadChannel(QProcess::ProcessChannel::StandardOutput);
@@ -386,7 +494,7 @@ void MznProcess::readStdOut()
     }
     while (p.canReadLine()) {
         auto line = QString::fromUtf8(p.readLine());
-        emit outputStdOut(line);
+        onStdOutLine(line);
     }
 }
 
@@ -396,7 +504,7 @@ void MznProcess::readStdErr()
     if (p.canReadLine()) {
         auto fragment = QString::fromUtf8(p.readAllStandardOutput());
         if (!fragment.isEmpty()) {
-            emit outputStdOut(fragment);
+            onStdOutLine(fragment);
         }
     }
     while (p.canReadLine()) {
@@ -412,7 +520,7 @@ void MznProcess::flushOutput()
 
     auto stdOut = QString::fromUtf8(p.readAllStandardOutput());
     if (!stdOut.isEmpty()) {
-        emit outputStdOut(stdOut);
+        onStdOutLine(stdOut);
     }
 
     auto stdErr = QString::fromUtf8(p.readAllStandardError());
@@ -421,131 +529,82 @@ void MznProcess::flushOutput()
     }
 }
 
-SolveProcess::SolveProcess(QObject* parent) :
-    MznProcess(parent)
+void MznProcess::onStdOutLine(const QString& line)
 {
-    connect(this, &MznProcess::outputStdOut, this, &SolveProcess::processStdout);
-    connect(this, &MznProcess::success, this, &SolveProcess::finished);
-    connect(this, &MznProcess::failure, this, &SolveProcess::finished);
-    connect(this, &MznProcess::finished, this, &SolveProcess::finished);
-}
+    emit outputStdOut(line);
 
-
-void SolveProcess::solve(const SolverConfiguration& sc, const QString& modelFile, const QStringList& dataFiles, const QStringList& extraArgs)
-{
-    state = State::Output;
-    outputBuffer.clear();
-    htmlBuffer.clear();
-    jsonBuffer.clear();
-
-    QFileInfo fi(modelFile);
-    QStringList args;
-    args << extraArgs << modelFile << dataFiles;
-    MznProcess::start(sc, args, fi.canonicalPath());
-}
-
-void SolveProcess::processStdout(QString line)
-{
-    auto trimmed = line.trimmed();
-    QRegExp jsonPattern("^(?:%%%(top|bottom))?%%%mzn-json(-init)?:(.*)");
-
-    // Can appear in any state
-    if (trimmed.startsWith("%%%mzn-stat")) {
-        if (!outputBuffer.isEmpty()) {
-            // For now, also process any output gragment so it appears in the correct order
-            emit fragment(outputBuffer.join(""));
-            outputBuffer.clear();
-        }
-        emit statisticOutput(line);
+    if (!parse) {
+        // Not running with --json-stream, so cannot parse output
+        emit unknownOutput(line);
         return;
-    } else if (trimmed.startsWith("%%%mzn-progress")) {
-        auto split = trimmed.split(" ");
-        bool ok = split.length() == 2;
-        if (ok) {
-            auto progress = split[1].toFloat(&ok);
-            if (ok) {
-                emit progressOutput(progress);
-                return;
-            }
-        }
     }
 
-    switch (state) {
-    case State::Output: // Normal solution output
-        if (trimmed == "----------") {
-            outputBuffer << line;
-            emit solutionOutput(outputBuffer.join(""));
-            if (!visBuffer.isEmpty()) {
-                emit jsonOutput(false, visBuffer);
-                visBuffer.clear();
+    QJsonParseError error;
+    auto json = QJsonDocument::fromJson(line.toUtf8(), &error);
+    if (!json.isNull()) {
+        auto msg = json.object();
+        auto msg_type = msg["type"].toString();
+        if (msg_type == "solution") {
+            auto sections = msg["output"].toObject().toVariantMap();
+            QStringList order;
+            if (msg["sections"].isArray()) {
+                for (auto it : msg["sections"].toArray()) {
+                    order << it.toString();
+                }
+            } else {
+                order = sections.keys();
             }
-            outputBuffer.clear();
-        } else if (trimmed == "==========" ||
-                   trimmed == "=====UNKNOWN=====" ||
-                   trimmed == "=====ERROR=====" ||
-                   trimmed == "=====UNSATISFIABLE=====" ||
-                   trimmed == "=====UNSATorUNBOUNDED=====" ||
-                   trimmed == "=====UNBOUNDED=====") {
-            outputBuffer << line;
-            emit finalStatus(outputBuffer.join(""));
-            outputBuffer.clear();
-        } else if (jsonPattern.exactMatch(trimmed)) {
-            auto captures = jsonPattern.capturedTexts();
-            state = captures[2] == "-init" ? State::JSONInit : State::JSON;
-            QString area = captures[1].isEmpty() ? "top" : captures[1];
-            auto jsonArea = area == "bottom" ? Qt::BottomDockWidgetArea : Qt::TopDockWidgetArea;
-            auto jsonPath = captures[3];
-            visBuffer << VisOutput(VisWindowSpec(jsonPath, jsonArea));
-        } else if (trimmed == "%%%mzn-html-start") {
-            state = State::HTML;
+            qint64 time = msg["time"].isDouble() ? static_cast<qint64>(msg["time"].toDouble()) : -1;
+            emit solutionOutput(sections, order, time);
+        } else if (msg_type == "checker") {
+            auto sections = msg["output"].toObject().toVariantMap();
+            QStringList order;
+            if (msg["sections"].isArray()) {
+                for (auto it : msg["sections"].toArray()) {
+                    order << it.toString();
+                }
+            } else {
+                order = sections.keys();
+            }
+            qint64 time = msg["time"].isDouble() ? static_cast<qint64>(msg["time"].toDouble()) : -1;
+            emit checkerOutput(sections, order, time);
+        } else if (msg_type == "status") {
+            qint64 time = msg["time"].isDouble() ? static_cast<qint64>(msg["time"].toDouble()) : -1;
+            emit finalStatus(msg["status"].toString(), time);
+        } else if (msg_type == "statistics") {
+            emit statisticsOutput(msg["statistics"].toObject().toVariantMap());
+        } else if (msg_type == "comment") {
+            auto comment = msg["comment"].toString();
+            emit commentOutput(comment);
+        } else if (msg_type == "time") {
+            qint64 time = msg["time"].isDouble() ? static_cast<qint64>(msg["time"].toDouble()) : -1;
+            emit timeOutput(time);
+        } else if (msg_type == "error") {
+            emit errorOutput(msg);
+        } else if (msg_type == "warning") {
+            emit warningOutput(msg);
+        } else if (msg_type == "progress") {
+            emit progressOutput(msg["progress"].toDouble());
+        } else if (msg_type == "paths") {
+            QVector<PathEntry> paths;
+            for (auto it : msg["paths"].toArray()) {
+                paths << it.toObject();
+            }
+            emit pathsOutput(paths);
+        } else if (msg_type == "profiling") {
+            QVector<TimingEntry> t;
+            for (auto it : msg["entries"].toArray()) {
+                t << it.toObject();
+            }
+            emit profilingOutput(t);
+        } else if (msg_type == "trace") {
+            emit traceOutput(msg["section"].toString(), msg["message"].toVariant());
         } else {
-            outputBuffer << line;
+            emit unknownOutput(line);
         }
-        break;
-    case State::HTML: // Seen %%%mzn-html-start
-        if (trimmed.startsWith("%%%mzn-html-end")) {
-            emit htmlOutput(htmlBuffer.join(""));
-            state = State::Output;
-            htmlBuffer.clear();
-        } else {
-            htmlBuffer << line;
-        }
-        break;
-    case State::JSONInit: // Seen %%%mzn-json-init
-        if (trimmed == "%%%mzn-json-end") {
-            visBuffer.last().data = jsonBuffer.join(" ");
-            state = State::Output;
-            jsonBuffer.clear();
-        } else if (trimmed == "%%%mzn-json-init-end") {
-            visBuffer.last().data = jsonBuffer.join(" ");
-            emit jsonOutput(true, visBuffer);
-            state = State::Output;
-            jsonBuffer.clear();
-            visBuffer.clear();
-        } else {
-            jsonBuffer << trimmed;
-        }
-        break;
-    case State::JSON: // Seen %%%mzn-json
-        if (trimmed == "%%%mzn-json-end") {
-            visBuffer.last().data = jsonBuffer.join(" ");
-            state = State::Output;
-            jsonBuffer.clear();
-        } else if (trimmed == "%%%mzn-json-time") {
-            // Wrap current buffer in array with elapsed time
-            jsonBuffer.prepend("[");
-            jsonBuffer << "," << QString().number(elapsedTime()) << "]";
-        } else {
-            jsonBuffer << trimmed;
-        }
-        break;
+        return;
     }
-}
 
-void SolveProcess::finished() {
-    // Make sure any remaining fragment is printed
-    if (!outputBuffer.empty()) {
-        emit fragment(outputBuffer.join(""));
-        outputBuffer.clear();
-    }
+    // Fall back to just printing everything
+    emit unknownOutput(line);
 }
